#include "lexer.h"
#include <iostream> // cout

using namespace std;

const char zero = 0;
const char quote = '"';

Lexer::Lexer (string fileName)
{
   f.open (fileName);
   if (! f.good ())
       error ("Cannot open file: " + fileName);

   ch = zero;
   nextChar ();
   nextToken ();
}

void Lexer::error (string msg)
{
    cout << "Error: " << msg << endl;
    exit (1);
}

void Lexer::nextChar()
{
    f >> ch;
    if (! f.good ())
       ch = zero;
}

void Lexer::nextToken()
{
    token = "";
    while (ch != zero && ch <= ' ')
        nextChar ();

    if (ch >= 'a' && ch <= 'z' || ch >= 'A' && ch <= 'Z')
    {
        kind = ident;
        while (ch >= 'a' && ch <= 'z' || ch >= 'A' && ch <= 'Z')
        {
            token = token + ch;
            nextChar ();
        }
    }
    else if (ch >= '0' && ch <= '9')
    {
        kind = number;
        while (ch >= '0' && ch <= '9')
        {
            token = token + ch;
            nextChar ();
        }
    }
    else if (ch == quote)
    {
        kind = text;
        nextChar (); // skip quote
        while (ch != quote && ch != zero)
        {
            token = token + ch;
            nextChar ();
        }
        nextChar (); // skip quote
    }
    else if (ch != zero)
    {
        kind = separator;
        token = string (1, ch);
        nextChar ();
    }
    else
    {
        kind = eos;
    }
}

bool Lexer::isSeparator (char c)
{
    return kind==separator && token==string(1, c);
}

void Lexer::checkSeparator(char c)
{
    if (!isSeparator(c)) error(string (1,c)+" expected");
    nextToken();
}

string Lexer:: readIdent ()
{
    if (kind!=ident)
        error("Identifier expected.");
    string result = token;
    nextToken();
    return result;
}

string Lexer:: readText ()
{
    if (kind!=text)
        error("Quoted string expected.");
    string result = token;
    nextToken();
    return result;
}
